module dff_timing_check(clk, reset_n, d, q);
output reg q;
input clk, d, reset_n;
specify
// define timing specparam values
specparam tSU = 10, tHD = 2, tPW = 25, tWPC = 10, tREC = 5;
// define module path delay rise and fall min:typ:max values
specparam tPLHclk = 4:6:9, tPHLclk = 5:8:11;
specparam tPLHreset = 3:5:6, tPHLreset = 4:7:9;
   // specify module path delays
   (clk *> q)     = (tPLHclk, tPHLclk);
   (reset_n *> q) = (tPLHreset, tPHLreset);
   // setup time : data to clock, only when reset_n is 1
   $setup(d, posedge clk &&& reset_n, tSU);
   // hold time: clock to data, only when reset_n is 1
   $hold(posedge clk, d &&& reset_n, tHD);
   // clock period check
   $period(posedge clk, tPW);
    // pulse width : reset_n
   $width(negedge reset_n, tWPC, 0);
    // recovery time: reset_n to clock
   $recovery(posedge reset_n, posedge clk, tREC);
endspecify
// the body of flip flop
always @(posedge clk or negedge reset_n)
   if (!reset_n) q <= 0; // reset_n signal is active-low.
   else          q <= d;
endmodule 